%date created on 1/12/05:
%This is a modified version of Jong's work20040812_viterbi.m
%Infer Viterbi path using dynamic HMM parameters.

function yy = HMM_viterbi_v5(precious, para);

warning off

yy = precious;
R = size(yy.hyb, 1);
len = size(yy.hyb, 2);

yy.mle = zeros(1, len) + NaN;

for c = para.select_contigs,
    %for c = 216:216
    select = yy.contigs(c,1):yy.contigs(c,2);

    o = yy.hyb(:, select);
    
    psnl = yy.psnl(select);
    pnl = yy.pnl(select);
    pfl = yy.pfl(select);
    pln = yy.pln(select);
    pll = yy.pll(select);

    mu_n = yy.mu_n(:, select);
    mu_f = yy.mu_f(:, select);
    mu_l = yy.mu_l(:, select);
    var_n = yy.var_n(:, select);
    var_f = yy.var_f(:, select);
    var_l = yy.var_l(:, select);

    if para.tie_para == 1,
        mu_n = ones(R, 1)*mu_n;
        mu_f = ones(R, 1)*mu_f;
        mu_l = ones(R, 1)*mu_l;
        var_n = ones(R, 1)*var_n;
        var_f = ones(R, 1)*var_f;
        var_l = ones(R, 1)*var_l;
    end

    % v(i,h) = log likelihood of the sequence of observations following time i,
    %          under the best possible explanation, if the hidden state at time
    %          i is h
    v = zeros(length(select), 18);
    v(:) = NaN;
    w = zeros(length(select), 18);

    % Base case.
    v(end,:) = 0;

    for i = (length(select)-1):-1:1

        transit0 = zeros(18, 18);
        for s = 1:5,
            transit0(s, s+1) = 1;
        end
        transit0(6, 7) = 1 - psnl(i);
        transit0(6, 18) = psnl(i);
        transit0(7, 8) = 1 - pnl(i);
        transit0(7, 18) = pnl(i);
        transit0(8, 18) = 1;
        for s = 9:16,
            transit0(s, s+1) = 1;
        end
        transit0(17, 17) = 1 - pfl(i);
        transit0(17, 18) = pfl(i);
        transit0(18, 1) = pln(i);
        transit0(18, 9) = 1 - pln(i) - pll(i);
        transit0(18, 18) = pll(i);

        for h = 1:18
            ll = log(transit0(h,:));
            for r = 1:R
                for h2 = 1:8
                    ll(h2) = ll(h2) - log(sqrt(2*pi*var_n(r, i+1)))...
                        - (o(r,i+1)-mu_n(r, i+1))^2 / (2*var_n(r, i+1));
                end
                for h2 = 9:17
                    ll(h2) = ll(h2) - log(sqrt(2*pi*var_f(r, i+1)))...
                        - (o(r,i+1)-mu_f(r, i+1))^2 / (2*var_f(r, i+1));
                end
                for h2 = 18
                    ll(h2) = ll(h2) - log(sqrt(2*pi*var_l(r, i+1)))...
                        - (o(r,i+1)-mu_l(r, i+1))^2 / (2*var_l(r, i+1));
                end
            end
            ll = ll + v(i+1,:);

            [v(i,h) w(i,h)] = max(ll);
        end
    end

    mle = zeros(size(select)) + NaN;
    ll = log(yy.posts(:,select(1))'); %seems better to set ll=0; 
                                      %although the result should be same 
    for r = 1:R
        for h = 1:8
            ll(h) = ll(h) - log(sqrt(2*pi*var_n(r, 1)))...
                - (o(r,1)-mu_n(r, 1))^2 / (2*var_n(r, 1));
        end
        for h = 9:17
            ll(h) = ll(h) - log(sqrt(2*pi*var_f(r, 1)))...
                - (o(r,1)-mu_f(r, 1))^2 / (2*var_f(r, 1));
        end
        for h = 18
            ll(h) = ll(h) - log(sqrt(2*pi*var_l(r, 1)))...
                - (o(r,1)-mu_l(r, 1))^2 / (2*var_l(r, 1));
        end
    end
    ll = ll + v(1,:);
    [x mle(1)] = max(ll);

    for i = 2:length(select)
        mle(i) = w(i-1,mle(i-1));
    end

    yy.mle(select) = mle;
end

