%modified on 1/20/05: Apply to simulated sloped hyb data
%date created 11/16/04: This module is the initial step for HMM model.
%Tasks:
% 1) set up a directory for HMM work
% 2) prepare data.

function data_module(parent_dir, job_id);

%create directory for job_id, if it doesn't exist. check data/parameter
% files are there.
execdir = [parent_dir, job_id, '\'];
if ~exist(execdir);
    coldstart = 1
    [status,message,messageid] = mkdir(parent_dir, job_id);
    if ~status,
        error(['Error creating working directiory: ', execdir]);
    end
    if ~exist([parent_dir, 'SetParameter.m'], 'file'),
        error(['Error finding SetParameter.m']);
    end
    [status,message,messageid] = copyfile([parent_dir, 'SetParameter.m'], ...
        [execdir, 'SetParameter.m'], 'f');
    if ~status,
        error(['Error creating SetParameter.m']);
    end
    mypwd = pwd;
    cd(execdir);
    para = SetParameter();
    cd(mypwd);
  
    hyb_data = [execdir, para.hyb_data];
    if ~exist(para.data_source, 'file'),
        error(['Error finding hyb data: ', para.data_source]);
    end
    
    %create subdirectory under job_id, which deposits HMM output
    [status,message,messageid] = mkdir([execdir, para.temp_datadir]);
    if ~status,
        error(['Error creating ', [execdir, para.temp_datadir]]);
    end

    %load data. Transform data as necessary. e.g., detrend blast/entropy
    %effect, PTT filter.
    load(para.data_source);

    if ~exist('Z', 'var'),
        error('Hyb data Z is not defined');
    else,
        precious.hyb = Z.hyb;
        if ~isfield(Z, 'contigs'),
            precious.contigs = [1 size(Z.hyb,2)];
        else,
            precious.contigs = Z.contigs;
        end
        if para.no_blast == 0,
            if ~isfield(Z, 'blast') | ~isfiled(Z, 'entropy'),
                error('Error: Must contain blast and entropy field for detrending');
            else,
                precious.blast = Z.blast;
                precious.entropy = Z.entropy;
            end
        end
    end

    %save data in job_id directory.
    save([execdir, para.hyb_data], 'precious');

else,
    coldstart = 0
    if ~exist([execdir, 'SetParameter.m'], 'file'),
        error(['Error finding SetParameter.m']);
    end
    mypwd = pwd;
    cd(execdir);
    para = SetParameter();
    cd(mypwd);
    hyb_data = [execdir, para.hyb_data];
    if ~exist(hyb_data, 'file'),
        error(['Error finding hyb data: ', hyb_data]);
    end

    %create subdirectory under job_id, which deposits HMM output
    if ~exist([execdir, para.temp_datadir]),
        [status,message,messageid] = mkdir([execdir, para.temp_datadir]);
        if ~status,
            error(['Error creating ', [execdir, para.temp_datadir]]);
        end
    end
end




