%date created 11/16/04: This module is the third and final step for HMM model.
%Tasks:
% 1) merge HMM outputs from different windows for each contig
% 2) merge HMM outputs for all contigs, obtaining HMM model parameters and
% posterior.
% 3) learn viterbi path using merged model parameters.
% Inputs:
%   execdir:  a unique id number for the task.
%   para:    a text file containing all the parameters necessary for the 
%            module.
function merge_module(execdir, para);
    
%merge HMM outputs
switch para.version,
    case 3,
        yy = HMM_merge_windows(execdir, para);
    case 4,
        yy = HMM_merge_windows_v4(execdir, para);
    case 5,
        yy = HMM_merge_windows_v5(execdir, para);
    otherwise,
        error('Invalid HMM version');
end

%learn viterbi path
switch para.version,
    case 3,
        precious = HMM_viterbi(yy, para);
    case 4,
        precious = HMM_viterbi_v4(yy, para);
    case 5,
        precious = HMM_viterbi_v5(yy, para);
    otherwise,
        error('Invalid HMM version');
end

save([execdir, para.hmm_result], 'precious');

